import { IRoomService } from '..';
import logger from '../../utils/common/logger';

interface IComponentManager {
  getComponentConfig(name: ComponentName): ComponentConfigItem;
  setComponentConfig(options: Partial<ComponentConfig>): boolean;
}

const componentNames = [
  'SwitchTheme',
  'LayoutControl',
  'RoomInfo',
  'RoomLink',
  'Language',
  'UserInfo',
  'ScreenShare',
  'FullScreen',
  'ManageMemberControl',
  'InviteControl',
  'ChatControl',
  'MoreControl',
] as const;

export type ComponentName = typeof componentNames[number];

type ComponentConfigItem = {
  visible: boolean;
  [key: string]: any;
};

export type ComponentConfig = {
  [key in ComponentName]: ComponentConfigItem;
};

const logPrefix = '[RoomService ComponentManager]';

export class ComponentManager implements IComponentManager {
  static instance?: ComponentManager;

  private service: IRoomService;

  constructor(service: IRoomService) {
    this.service = service;
  }


  // Initialize componentConfig with default visibility settings
  private componentConfig: ComponentConfig = Object.fromEntries(componentNames.map(name => [name, {
    visible: true,
  }])) as ComponentConfig;

  static getInstance(ctx: IRoomService): ComponentManager {
    if (!ComponentManager.instance) {
      ComponentManager.instance = new ComponentManager(ctx);
    }
    return ComponentManager.instance;
  }

  static destroyInstance(): void {
    if (!ComponentManager.instance) return;
    ComponentManager.instance = undefined;
  }

  public getComponentConfig(name: ComponentName) {
    return this.componentConfig[name];
  }

  public setComponentConfig(options: Partial<ComponentConfig>) {
    try {
      Object.entries(options).forEach(([name, current]) => {
        this.componentConfig[name as ComponentName] = Object.assign(
          {},
          this.componentConfig[name as ComponentName],
          current,
        );
      });
      return true;
    } catch (error) {
      logger.error(`${logPrefix}setComponentConfig error: ${error}`);
      return false;
    }
  }
}
